package com.product.kt.test.service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.product.core.entity.FieldSetEntity;


@Service
public class ToolsThread {

	private static final Logger LOG = LoggerFactory.getLogger(ToolsThread.class);
	 
	@Autowired
	SyncToolsHandler syncToolsHandler;
 
	@Autowired
	ReadToolsService readToolsService;
	
	
	
	
	
	/**
	 * 
	 */
	public void receiveBookJobRun() {
 
		//数据集
		int countSize=readToolsService.getToolCount();
		//每页大小
		int pageSize=5000;
		//总页数
		int pageCount=countSize/pageSize+1;
		
		// 入库开始时间
		Long inserOrUpdateBegin = System.currentTimeMillis();
		LOG.info("数据更新开始时间:" + inserOrUpdateBegin);
		
		for (int k = 0; k < pageCount; k++) {
			int index=pageSize*k;
			
			//获取分页数据
			List<FieldSetEntity> readToolModels = readToolsService.getToolListByAutoId(index, pageSize);
	 
			// 接收集合各段的 执行的返回结果
			List<Future<String>> futureList = new ArrayList<Future<String>>();
			// 集合总条数
			if (readToolModels != null) {
				// 将集合切分的段数(2*CPU的核心数)
				int threadSum = 2 * Runtime.getRuntime().availableProcessors();
				int listSize = readToolModels.size();
				int listStart, listEnd;
				// 当总条数不足threadSum条时 用总条数 当做线程切分值
				if (threadSum > listSize) {
					threadSum = listSize;
				}
	 
				// 将list 切分多份 多线程执行
				for (int i = 0; i < threadSum; i++) {
					// 计算切割 开始和结束
					listStart = listSize / threadSum * i;
					listEnd = listSize / threadSum * (i + 1);
					// 最后一段线程会 出现与其他线程不等的情况
					if (i == threadSum - 1) {
						listEnd = listSize;
					}
					// 数据切断
					List<FieldSetEntity> readToolList= readToolModels.subList(listStart, listEnd);
	 
					// 每段数据集合并行入库
					futureList.add(syncToolsHandler.syncTools(readToolList, i));
				}
	 
				// 对各个线程段结果进行解析
				for (Future<String> future : futureList) {
					if (null != future) {
						try {
							String str = future.get().toString();
							LOG.info("current thread id =" + Thread.currentThread().getName() + ",result=" + str);
	 
						} catch (ExecutionException | InterruptedException e) {
	 
							LOG.info("线程运行异常！");
						}
	 
					} else {
						LOG.info("线程运行异常！");
					}
				}
			}
	 
		}
		Long inserOrUpdateEnd = System.currentTimeMillis();
		LOG.info("数据更新结束时间:" + inserOrUpdateEnd + "。此次更新数据花费时间为：" + (inserOrUpdateEnd - inserOrUpdateBegin));
	}
}
